#include "Image.hh"
#include "couleur.hh"
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

void DecomposeRGB(Image<int>& p_coul,Image<int>& p_R,Image<int>& p_G,Image<int>& p_B) {
  int index,i,j;
  int h = p_coul.PL();
  int w = p_coul.PW();
  int* COUL = p_coul.PI();
  int* PR = p_R.PI();
  int* PG = p_G.PI();
  int* PB = p_B.PI();
  
  index = 0;
  for (j=0;j<h;j++)
    for (i=0;i<w;i++) {
      PR[index] = COUL[index]%256;
      PG[index] = (COUL[index]>>8)%256;
      PB[index] = (COUL[index]>>16)%256;
      index++;
    }
}

void RecomposeRGB(Image<int>& p_coul,Image<int>& p_R,Image<int>& p_G,Image<int>& p_B) {
  int index,i,j;
  int h = p_coul.PL();
  int w = p_coul.PW();
  int* COUL = p_coul.PI();
  int* PR = p_R.PI();
  int* PG = p_G.PI();
  int* PB = p_B.PI();
  
  index = 0;
  for (j=0;j<h;j++)
    for (i=0;i<w;i++) {
      COUL[index] = PR[index] + (PG[index]<<8) + (PB[index]<<16);
      index++;
    }
}


void DecomposeHSV(Image<int>& p_coul,Image<int>& p_H,Image<int>& p_S,Image<int>& p_V) {
  int index,i,j;
  int h = p_coul.PL();
  int w = p_coul.PW();
  int* COUL = p_coul.PI();
  int* PH = p_H.PI();
  int* PS = p_S.PI();
  int* PV = p_V.PI();
  int r,g,b;
  int m,max,min,med,sat;
  double dsat,dm,dmax,dmin,dmed;
  int lambda,mult;
  
  index = 0;
  for (j=0;j<h;j++)
    for (i=0;i<w;i++) {
      r = COUL[index]%256;
      g = (COUL[index]>>8)%256;
      b = (COUL[index]>>16)%256;
      //Calcul de la moyenne m
      m = (r+b+g)/3;
      //Calcul des statistiques d'ordre {min,med,max}
      //Calcul du numro de sextant (lambda)
      if (r<g) {
	if (g<b) {
	  min=r;med=g;max=b;lambda=3;mult=-1;
	} else {
	  if (r<b) {
	    min=r;med=b;max=g;lambda=2;mult=1;
	  } else {
	    min=b;med=r;max=g;lambda=1;mult=-1;
	  } 
	}
      } else {
	if (r<b) {
	  min=g;med=r;max=b;lambda=4;mult=1;
	} else {
	  if (g<b) {
	    min=g;med=b;max=r;lambda=5;mult=-1;
	  } else {
	    min=b;med=g;max=r;lambda=0;mult=1;
	  }
	}
      }
      // Calcul de l'intensit
      PV[index] = m;
      // Calcul de la saturation
      if (m>med) PS[index] = (int)((3*(max-m))/2);
      else PS[index] = (int)((3*(m-min))/2);
      // Calcul de la teinte
      dm = ((double)m)/255;
      dmin = ((double)min)/255;
      dmed = ((double)med)/255;
      dmax = ((double)max)/255;
      if (dm>dmed) dsat = 3*(dmax-dm)/2;
      else dsat = 3*(dm-dmin)/2;
      if (dsat =! 0) {
	PH[index] = (int)(255*(lambda + 0.5 - mult*((dmax+dmin-2*dmed)/(2*dsat)))/6);
      } else PH[index]=0;
      index++;
    }
}

void DecomposeYUV(Image<int>& p_coul,Image<int>& p_Y,Image<int>& p_U,Image<int>& p_V) {
  int index,i,j;
  int h = p_coul.PL();
  int w = p_coul.PW();
  int* COUL = p_coul.PI();
  int* PY = p_Y.PI();
  int* PU = p_U.PI();
  int* PV = p_V.PI();
  float r,g,b;
  float Y_f,Cr_f,Cb_f;

   index = 0;
   for (j=0;j<h;j++)
     for (i=0;i<w;i++) {
       r = (float)(COUL[index]%256);
       g = (float)((COUL[index]>>8)%256);
       b = (float)((COUL[index]>>16)%256);
       //Calcul de la luminance Y
       Y_f = 0.2989*r + 0.5866*g + 0.1145*b;
       //Calcul du contraste Bleu-Jaune Cb
       Cb_f = -0.1687*r - 0.3312*g + 0.5*b;
       //Calcul du contraste Rouge-Cyan Cr
       Cr_f = 0.5*r - 0.4183*g - 0.0817*b;
       PY[index] = (int)(Y_f);
       PU[index] = 128 + (int)(Cb_f);
       PV[index] = 128 + (int)(Cr_f);
       index++;
     }
}
